#include "motion.h"
#include "conf.h"
#include "xmlrpc-httpd.h"

#ifdef HAVE_XMLRPC

void info(struct xmlrpc_httpd_call *call, xmlrpc_value *param_array, void *userdata)
{
	struct context **cnt=userdata;
	xmlrpc_int32 i=0;
	xmlrpc_env env;
	
	while (cnt[i++]);
	i--;

	xmlrpc_env_init(&env);
	
	xmlrpc_httpd_result(&env, call, xmlrpc_build_value(&env, "(si)", "Motion "VERSION, i));
}

extern config_param config_params[];

void conf_write(struct xmlrpc_httpd_call *call, xmlrpc_value *param_array, void *userdata)
{
	struct context **cnt=userdata;
	xmlrpc_env env;
	
	conf_print(cnt);

	xmlrpc_env_init(&env);
	
	xmlrpc_httpd_result(&env, call, xmlrpc_build_value(&env, "()"));
}

void conf_list(struct xmlrpc_httpd_call *call, xmlrpc_value *param_array, void *userdata)
{
	int j;
	xmlrpc_env env;
	xmlrpc_value *config_struct;
	xmlrpc_value *ret;

	xmlrpc_env_init(&env);

	ret = xmlrpc_build_value (&env, "()");
	j=0;
	while (config_params[j].param_name != NULL) {
		if (!config_params[j].conf_value)
			continue;
		config_struct = xmlrpc_build_value (&env, "{s:s,s:s,s:s}", "option", config_params[j].param_name, "parameter", config_type(&config_params[j]), "help", config_params[j].param_help);
		xmlrpc_array_append_item(&env, ret, config_struct);
		j++;
	}

	xmlrpc_httpd_result(&env, call, ret);
}

void conf_get(struct xmlrpc_httpd_call *call, xmlrpc_value *param_array, void *userdata)
{
	struct context **cnt=userdata;
	int i=0, j=0, thread;
	xmlrpc_env env;
	char *param;
	char *res=NULL;
	
	while (cnt[++i]);

	xmlrpc_env_init(&env);
	xmlrpc_parse_value(&env, param_array, "(is)", &thread, &param);
	printf("conf.get: %d %s\n", thread, param);

	while (config_params[j].param_name != NULL) {
		if (!strcasecmp(param, config_params[j].param_name))
			break;
		j++;
	}

	if (thread>=0 && thread<i && config_params[j].param_name)
		res=config_params[j].print(cnt, NULL, j, thread);
	if (!res)
		res="";
	xmlrpc_httpd_result(&env, call, xmlrpc_build_value(&env, "s", res));
}

void conf_set(struct xmlrpc_httpd_call *call, xmlrpc_value *param_array, void *userdata)
{
	struct context **cnt=userdata;
	int i=0, thread;
	xmlrpc_env env;
	char *param, *arg;
	
	while (cnt[++i]);

	xmlrpc_env_init(&env);
	xmlrpc_parse_value(&env, param_array, "(iss)", &thread, &param, &arg);
	printf("conf.set: %d %s\n", thread, param);

	if (thread>=0 && thread<i) {
		conf_cmdparse(cnt+thread, param, arg);
		xmlrpc_httpd_result(&env, call, xmlrpc_build_value(&env, "()"));
	}
}

void action_makemovie(struct xmlrpc_httpd_call *call, xmlrpc_value *param_array, void *userdata)
{
	struct context **cnt=userdata;
	int i=0, thread;
	xmlrpc_env env;

	while (cnt[++i]);

	xmlrpc_env_init(&env);
	xmlrpc_parse_value(&env, param_array, "(i)", &thread);

	if (thread>=0 && thread<i) {
		cnt[thread]->makemovie=1;
	}
	xmlrpc_httpd_result(&env, call, xmlrpc_build_value(&env, "()"));
}

void action_snapshot(struct xmlrpc_httpd_call *call, xmlrpc_value *param_array, void *userdata)
{
	struct context **cnt=userdata;
	int i=0, thread;
	xmlrpc_env env;

	while (cnt[++i]);

	xmlrpc_env_init(&env);
	xmlrpc_parse_value(&env, param_array, "(i)", &thread);

	if (thread>=0 && thread<i) {
		cnt[thread]->snapshot=1;
	}
	xmlrpc_httpd_result(&env, call, xmlrpc_build_value(&env, "()"));
}

void detection_pause(struct xmlrpc_httpd_call *call, xmlrpc_value *param_array, void *userdata)
{
	struct context **cnt=userdata;
	int i=0, thread;
	xmlrpc_env env;

	while (cnt[++i]);

	xmlrpc_env_init(&env);
	xmlrpc_parse_value(&env, param_array, "(i)", &thread);

	if (thread>=0 && thread<i) {
		cnt[thread]->pause=1;
	}
	xmlrpc_httpd_result(&env, call, xmlrpc_build_value(&env, "()"));
}

void detection_resume(struct xmlrpc_httpd_call *call, xmlrpc_value *param_array, void *userdata)
{
	struct context **cnt=userdata;
	int i=0, thread;
	xmlrpc_env env;

	while (cnt[++i]);

	xmlrpc_env_init(&env);
	xmlrpc_parse_value(&env, param_array, "(i)", &thread);

	if (thread>=0 && thread<i) {
		cnt[thread]->pause=0;
	}
	xmlrpc_httpd_result(&env, call, xmlrpc_build_value(&env, "()"));
}

void *motion_control(void *arg)
{
	struct context **cnt=arg;
	
	xmlrpc_httpd_method_add("motion.info", info, (void *)cnt);
	xmlrpc_httpd_method_add("motion.conf.list", conf_list, (void *)cnt);
	xmlrpc_httpd_method_add("motion.conf.get", conf_get, (void *)cnt);
	xmlrpc_httpd_method_add("motion.conf.set", conf_set, (void *)cnt);
	xmlrpc_httpd_method_add("motion.conf.write", conf_write, (void *)cnt);
	xmlrpc_httpd_method_add("motion.action.makemovie", action_makemovie, (void *)cnt);
	xmlrpc_httpd_method_add("motion.action.snapshot", action_snapshot, (void *)cnt);
	xmlrpc_httpd_method_add("motion.detection.pause", detection_pause, (void *)cnt);
	xmlrpc_httpd_method_add("motion.detection.resume", detection_resume, (void *)cnt);

	xmlrpc_httpd_run(cnt[0]);
	return 0;
}

#endif /* HAVE_XMLRPC */

